/**
 * @file FigureUtil.cpp
 * @brief LN^tBMA֘A[eBeB֐
 * @author cherub
 */
#include "stdafx.h"
#include <cmath>
#include "FigureUtil.h"
#include "Matrix.h"

/**
 * @fn float calc_pose_distance(const std::vector<D3DXVECTOR3> &pc0, const std::vector<D3DXVECTOR3> &pc1)
 * @brief 2̊֐߈ʒu_Qf[^̃[NbȟvZ
 * @param pc0 [in] _Q0
 * @param pc1 [in] _Q1
 * @return [Nbh
 */
float calc_pose_distance(const std::vector<D3DXVECTOR3> &pc0, const std::vector<D3DXVECTOR3> &pc1)
{
	D3DXMATRIX trans = NFigureUtil::CalcTransformBetweenFigures(pc0, pc1);

	D3DXVECTOR3 diff;
	float dist =0.0f;
	for (size_t j = 0;j < pc0.size();++j)
	{
		D3DXVec3TransformCoord(&diff, &pc1[j], &trans);
		diff = pc0[j] - diff;
		dist += D3DXVec3LengthSq(&diff);
	}

	return sqrtf(dist / static_cast<float>(pc0.size()));
}

/**
 * @fn D3DXMATRIX NFigureUtil::CalcTransformBetweenFigures(const std::vector<D3DXVECTOR3> &pc0, const std::vector<D3DXVECTOR3> &pc1)
 * @brief 2̓_Q̊Ԃ̃[NbhaŏgXtH[̌vZ: min dist(pc0, trans(pc1))
 * @param pc0 [in] _Q0
 * @param pc1 [in] _Q1
 * @return gXtH[s
 */
D3DXMATRIX NFigureUtil::CalcTransformBetweenFigures(const std::vector<D3DXVECTOR3> &pc0, const std::vector<D3DXVECTOR3> &pc1)
{
	float top_left = 0, top_right = 0, bottom_left = 0, bottom_right = 0;
	float ax1 = 0, ax2 = 0, az1 = 0, az2 = 0;
	float tx, tz, ty = 0, ry;
	const float fnum = static_cast<float>(pc0.size());

	for (size_t i = 0;i < pc0.size();++i)
	{
		top_left += pc0[i].x * pc1[i].z - pc1[i].x * pc0[i].z;
		bottom_left += pc0[i].x * pc1[i].x + pc0[i].z * pc1[i].z;

		ax1 += pc0[i].x;
		az1 += pc0[i].z;
		ax2 += pc1[i].x;
		az2 += pc1[i].z;

		ty += pc0[i].y - pc1[i].y;
	}

	top_right = (ax1 * az2 - ax2 * az1) / fnum;
	bottom_right = (ax1 * ax2 + az1 * az2) / fnum;

	ry = std::atan2f(top_left - top_right, bottom_left - bottom_right);
	tx = (ax1 - ax2 * cosf(ry) - az2 * sinf(ry)) / fnum;
	tz = (az1 + ax2 * sinf(ry) - az2 * cosf(ry)) / fnum;
	ty /= fnum;

	D3DXMATRIX mr, mt;
	D3DXMatrixRotationY(&mr, ry);
	D3DXMatrixTranslation(&mt, tx, ty, tz);
	return mr * mt;
}

/**
 * @fn float NFigureUtil::PoseDistance(const CFigure &fig0, const CFigure &fig1)
 * @brief 2̃tBMA̎px 
 * @param fig0 [in] tBMA0
 * @param fig1 [in] tBMA1
 * @return px
 */
float NFigureUtil::PoseDistance(const CFigure &fig0, const CFigure &fig1)
{
	std::vector<D3DXVECTOR3> pc0 = fig0.GetPointCloud();
	std::vector<D3DXVECTOR3> pc1 = fig1.GetPointCloud();
	return calc_pose_distance(pc0, pc1);
}

/**
 * @fn float NFigureUtil::PoseDistance(CFigure &fig, const CMotionData &mot, size_t frm0, size_t frm1)
 * @brief 2̃t[ł̎px
 * @param fig [in/out] tBMA
 * @param mot [in] [Vf[^
 * @param frm0 [in] t[0
 * @param frm1 [in] t[1
 * @return px
 */
float NFigureUtil::PoseDistance(CFigure &fig, const CMotionData &mot, size_t frm0, size_t frm1)
{
	fig.SetPose(&mot, frm0);
	std::vector<D3DXVECTOR3> pc0 = fig.GetPointCloud();
	fig.SetPose(&mot, frm1);
	std::vector<D3DXVECTOR3> pc1 = fig.GetPointCloud();
	return calc_pose_distance(pc0, pc1);
}
