/**
 * @file MotionData.cpp
 * @brief [Vf[^NX
 * @author cherub
 */
#include "StdAfx.h"
#include "MotionData.h"

/**
 * @fn CMotionData::CMotionData(void)
 * @brief ftHgRXgN^
 */
CMotionData::CMotionData(void)
{
	m_nJoints = NULL;
	m_nFrames = NULL;
	m_pPosition = NULL;
	m_pRotation = NULL;
}

/**
 * @fn CMotionData::CMotionData(size_t frames, size_t joints)
 * @brief qtRXgN^
 * @param frames [in] t[
 * @param joints [in] ֐ߐ
 */
CMotionData::CMotionData(size_t frames, size_t joints)
{
	m_nJoints = NULL;
	m_nFrames = NULL;
	m_pPosition = NULL;
	m_pRotation = NULL;
	Initialize(frames, joints);
}

/**
 * @fn CMotionData::CMotionData(const CMotionData &src)
 * @brief Rs[RXgN^
 * @param src [in] Rs[
 */
CMotionData::CMotionData(const CMotionData &src)
{
	m_nJoints = NULL;
	m_nFrames = NULL;
	m_pPosition = NULL;
	m_pRotation = NULL;

	Initialize(src.NumFrames(), src.NumJoints());
	::memcpy(m_pPosition, src.m_pPosition, sizeof(D3DXVECTOR3) * NumFrames());
	::memcpy(m_pRotation, src.m_pRotation, sizeof(D3DXQUATERNION) * NumFrames() * NumJoints()); 
}

/**
 * @fn CMotionData::~CMotionData(void)
 * @brief fXgN^
 */
CMotionData::~CMotionData(void)
{
	Destroy();
}

/**
 * @fn bool CMotionData::Initialize(size_t frames, size_t joints)
 * @brief ֐
 * @param frames [in] t[
 * @param joints [in] ֐ߐ
 * @retval true 
 * @retval false s
 */
bool CMotionData::Initialize(size_t frames, size_t joints)
{
	Destroy();
	return Allocate(frames, joints);
}

/**
 * @fn CMotionData& CMotionData::operator =(const CMotionData &src)
 * @brief Zq
 * @param src [in] 
 * @return gւ̎Q
 */
CMotionData& CMotionData::operator =(const CMotionData &src)
{
	if (!Initialize(src.NumFrames(), src.NumJoints()))
		return *this;

	::memcpy(m_pPosition, src.m_pPosition, sizeof(D3DXVECTOR3) * NumFrames());
	::memcpy(m_pRotation, src.m_pRotation, sizeof(D3DXQUATERNION) * NumFrames() * NumJoints()); 
	return *this;
}

/**
 * @fn bool CMotionData::Allocate(size_t frames, size_t joints)
 * @brief mۊ֐
 * @param frames [in] t[
 * @param joints [in] ֐ߐ
 * @retval true mې
 * @retval false mێs
 */
bool CMotionData::Allocate(size_t frames, size_t joints)
{
	m_pPosition = new D3DXVECTOR3[frames];
	m_pRotation = new D3DXQUATERNION[frames * joints];
	if (m_pRotation == NULL || m_pPosition == NULL)
	{
		Destroy();
		return false;
	}

	m_nJoints = joints;
	m_nFrames = frames;

	D3DXVECTOR3 zerov;
	D3DXQUATERNION iq(0, 0, 0, 1);
	for (size_t f = 0;f < m_nFrames;++f)
	{
		POS(f) = zerov;
		for (size_t j = 0;j < m_nJoints;++j)
			ROT(f, j) = iq;
	}
	return true;
}

/**
 * @fn void CMotionData::Destroy()
 * @brief ֐
 */
void CMotionData::Destroy()
{
	if (m_pPosition != NULL)
	{
		delete[] m_pPosition;
		m_pPosition = NULL;
	}
	if (m_pRotation != NULL)
	{
		delete[] m_pRotation;
		m_pRotation = NULL;
	}
	m_nJoints = 0;
	m_nFrames = 0;
}

/**
 * @fn bool CMotionData::IsActive() const
 * @brief f[^p\
 * @retval true f[^p
 * @retval false f[^ps
 */
bool CMotionData::IsActive() const
{
	return m_pPosition != NULL && m_pRotation != NULL;
}

/**
 * @fn bool CMotionData::SetRotation(size_t frm, size_t jid, const D3DXQUATERNION &q)
 * @brief ֐߉]ʂ̐ݒiNH[^jIj
 * @param frm [in] t[ԍ
 * @param jid [in] ֐߃CfNX
 * @param q [in] ]NH[^jI
 * @retval true ݒ萬
 * @retval false ݒ莸s
 */
bool CMotionData::SetRotation(size_t frm, size_t jid, const D3DXQUATERNION &q)
{
	if (!IsRange(frm, jid))
		return false;
	ROT(frm, jid) = q;
	return true;
}

/**
 * @fn bool CMotionData::SetRotation(size_t frm, size_t jid, const D3DXMATRIX &m)
 * @brief ֐߉]ʂ̐ݒi]sj
 * @param frm [in] t[ԍ
 * @param jid [in] ֐߃CfNX
 * @param m [in] ]s
 * @retval true ݒ萬
 * @retval false ݒ莸s
 */
bool CMotionData::SetRotation(size_t frm, size_t jid, const D3DXMATRIX &m)
{
	if (!IsRange(frm, jid))
		return false;

	D3DXQuaternionRotationMatrix(&ROT(frm, jid), &m);
	return true;
}

/**
 * @fn bool CMotionData::SetPosition(size_t frm, const D3DXVECTOR3 &v)
 * @brief [gʒu̐ݒ
 * @param frm [in] t[ԍ
 * @param v [in] ʒuxNg
 * @retval true ݒ萬
 * @retval false ݒ莸s
 */
bool CMotionData::SetPosition(size_t frm, const D3DXVECTOR3 &v)
{
	if (!IsRange(frm))
		return false;
	POS(frm) = v;
	return true;
}

/**
 * @fn D3DXQUATERNION CMotionData::GetRotation(size_t frm, size_t jid) const
 * @brief ֐߉]NH[^jI̎擾
 * @param frm [in] t[ԍ
 * @param jid [in] ֐߃CfNX
 * @return ֐߉]NH[^jI
 */
D3DXQUATERNION CMotionData::GetRotation(size_t frm, size_t jid) const
{
	if (!IsRange(frm, jid))
		return D3DXQUATERNION();
	return ROT(frm, jid);
}

/**
 * @fn D3DXMATRIX CMotionData::GetRotationMatrix(size_t frm, size_t jid) const
 * @brief ֐߉]s̎擾
 * @param frm [in] t[ԍ
 * @param jid [in] ֐߃CfNX
 * @return ֐߉]s
 */
D3DXMATRIX CMotionData::GetRotationMatrix(size_t frm, size_t jid) const
{
	D3DXMATRIX m;
	if (!IsRange(frm, jid))
		return m;
	D3DXMatrixRotationQuaternion(&m, &ROT(frm, jid));
	return m;
}

/**
 * @fn D3DXVECTOR3 CMotionData::GetPosition(size_t frm) const
 * @brief [gʒuxNg̎擾
 * @param frm [in] t[ԍ
 * @return [gʒuxNg
 */
D3DXVECTOR3 CMotionData::GetPosition(size_t frm) const
{
	if (!IsRange(frm))
		return D3DXVECTOR3();
	return POS(frm);
}

/**
 * @fn D3DXMATRIX CMotionData::GetPositionMatrix(size_t frm) const
 * @brief [gʒus̎擾
 * @param frm [in] t[ԍ
 * @return [gʒus
 */
D3DXMATRIX CMotionData::GetPositionMatrix(size_t frm) const
{
	D3DXMATRIX m;
	if (!IsRange(frm))
		return m;
	D3DXVECTOR3 v = POS(frm);
	D3DXMatrixTranslation(&m, v.x, v.y, v.z);
	return m;
}
