// SceneView.cpp : CSceneView NX̎
//

#include "stdafx.h"
#include "Parametrization.h"

#include "MainDoc.h"
#include "SceneView.h"

#include <gsl/gsl_math.h>
#include "FigureUtil.h"
#include "MotionFile.h"
#include "MotionData.h"
#include "MotionSignal.h"
#include "DistanceMatrix.h"
#include "TimewarpMatrix.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CSceneView

IMPLEMENT_DYNCREATE(CSceneView, CView)

BEGIN_MESSAGE_MAP(CSceneView, CView)
	ON_MESSAGE(WM_USER_RENDER_SCENE, OnRenderScene)
	ON_WM_DESTROY()
	ON_WM_PAINT()
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_MBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_MOUSEWHEEL()
	ON_WM_TIMER()
	ON_COMMAND(ID_FILE_OPEN, &CSceneView::OnFileOpen)
	ON_COMMAND(ID_FILE_PLAY, &CSceneView::OnFilePlay)
	ON_UPDATE_COMMAND_UI(ID_FILE_PLAY, &CSceneView::OnUpdateFilePlay)
	ON_COMMAND(ID_FILE_SAVE_AS, &CSceneView::OnFileSaveAs)
	ON_COMMAND(ID_MOTION_SPATIALALIGN, &CSceneView::OnMotionSpatialAlign)
	ON_COMMAND(ID_MOTION_TEMPORALALIGN, &CSceneView::OnMotionTemporalAlign)
	ON_COMMAND(ID_MOTION_CALC_PARAMETERS, &CSceneView::OnMotionCalcParameters)
	ON_COMMAND(ID_MOTION_ANALYZE, &CSceneView::OnMotionAnalyze)
	ON_WM_KEYDOWN()
	ON_COMMAND(ID_MOTION_SYNTHESIS, &CSceneView::OnMotionSynthesis)
END_MESSAGE_MAP()

// CSceneView RXgNV/fXgNV

CSceneView::CSceneView()
{
	m_pD3D = NULL;
	m_pDeviceD3D = NULL;
	m_pEffect = NULL;

	m_hWorld = NULL;
	m_hWorldIT = NULL;
	m_hWorldViewProj = NULL;
	m_hEyePos = NULL;

	m_pvView[0].x = 20.0f;
	m_pvView[0].y = 0.0f;
	m_pvView[0].z = 0.0f;

	m_pvView[1].x = 0.0f;
	m_pvView[1].y = 0.0f;
	m_pvView[1].z = 0.0f;

	m_pvView[2].x = 0.0f;
	m_pvView[2].y = 1.0f;
	m_pvView[2].z = 0.0f;

	for (int i = 0;i < 3;i++)
		D3DXMatrixIdentity(&m_pmViewTrans[i]);

	m_pMeshSphere = NULL;

	m_pFigure = NULL;
	m_pMotion = NULL;
	m_pKrigingSystem = NULL;

	m_vecSampleFigures.clear();
	m_vecSampleMotions.clear();
	m_vecSampleITW.clear();
	m_vecSampleParams.clear();
	m_vControlParam.Initialize(2);
	m_vParaRange[0].Initialize(2);
	m_vParaRange[1].Initialize(2);

	m_frame = 0;
	m_bPlay = false;
}

CSceneView::~CSceneView()
{
}

BOOL CSceneView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: ̈ʒu CREATESTRUCT cs C Window NX܂̓X^C
	//  CĂB

	return CView::PreCreateWindow(cs);
}

// CSceneView `

void CSceneView::OnDraw(CDC* /*pDC*/)
{
	CMainDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	if (!pDoc)
		return;

	// TODO: ̏ꏊɃlCeBu f[^p̕`R[hǉ܂B
}


// CSceneView ff

#ifdef _DEBUG
void CSceneView::AssertValid() const
{
	CView::AssertValid();
}

void CSceneView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CMainDoc* CSceneView::GetDocument() const // fobOȊÕo[W̓CCłB
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMainDoc)));
	return (CMainDoc*)m_pDocument;
}
#endif //_DEBUG


// CSceneView bZ[W nh

void CSceneView::OnInitialUpdate()
{
	CView::OnInitialUpdate();

	HRESULT hr;

	if (hr = InitDirectGraphics(), FAILED(hr))
	{
		MessageBox(DXGetErrorString9(hr));
		exit(hr);
	}

	if (hr = InitRenderState(), FAILED(hr))
	{
		MessageBox(DXGetErrorString9(hr));
		exit(hr);
	}

	if (hr = InitHLSL(), FAILED(hr))
	{
		MessageBox(DXGetErrorString9(hr));
		exit(hr);
	}

	D3DXCreateSphere(m_pDeviceD3D, 0.5f, 10, 10, &m_pMeshSphere, NULL);

	SetTimer(0, 30, NULL);
}

void CSceneView::ReleaseMotions()
{
	if (m_pFigure != NULL)
	{
		delete m_pFigure;
		m_pFigure = NULL;
	}
	if (m_pMotion != NULL)
	{
		delete m_pMotion;
		m_pMotion = NULL;
	}
	if (m_pKrigingSystem != NULL)
	{
		delete m_pKrigingSystem;
		m_pKrigingSystem = NULL;
	}
	while (!m_vecSampleFigures.empty())
	{
		delete m_vecSampleFigures.back();
		m_vecSampleFigures.pop_back();
	}
	while (!m_vecSampleMotions.empty())
	{
		delete m_vecSampleMotions.back();
		m_vecSampleMotions.pop_back();
	}
	while (!m_vecSampleITW.empty())
	{
		delete m_vecSampleITW.back();
		m_vecSampleITW.pop_back();
	}
	while (!m_vecSampleParams.empty())
	{
		delete m_vecSampleParams.back();
		m_vecSampleParams.pop_back();
	}

	m_vParaRange[0].Initialize(2);
	m_vParaRange[1].Initialize(2);
	m_vControlParam.Initialize(2);
}

void CSceneView::OnDestroy()
{
	CView::OnDestroy();

	ReleaseMotions();

	if (m_pMeshSphere != NULL)
		m_pMeshSphere->Release();
	if (m_pEffect != NULL)
		m_pEffect->Release();
	if (m_pDeviceD3D != NULL)
		m_pDeviceD3D->Release();
	if (m_pD3D != NULL)
		m_pD3D->Release();
}

void CSceneView::OnPaint()
{
	CPaintDC dc(this);

	SendMessage(WM_USER_RENDER_SCENE);
}

void CSceneView::OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);

	if (m_pD3D == NULL)
		return;

	D3DDISPLAYMODE d3dDM;
	HRESULT hr;

	if (hr = m_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3dDM), FAILED(hr))
		return;

	float fAspect = ((float)cx / (float)cy) * ((d3dDM.Height * 4.0f) / (d3dDM.Width * 3.0f));
	D3DXMatrixPerspectiveFovLH(&m_mProjection, D3DX_PI / 4.0f, fAspect, 1.0f, 1000.0f);
}

HRESULT CSceneView::InitDirectGraphics(void)
{
	HRESULT hr;
	D3DDISPLAYMODE d3dDM;
	D3DPRESENT_PARAMETERS d3dPP;

	if (m_pD3D = Direct3DCreate9(D3D_SDK_VERSION), m_pD3D == NULL)
		return E_FAIL;

	if (hr = m_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3dDM), FAILED(hr))
		return hr;

	ZeroMemory(&d3dPP, sizeof(D3DPRESENT_PARAMETERS));
	d3dPP.Windowed = TRUE;
	d3dPP.BackBufferWidth = 640;
	d3dPP.BackBufferHeight = 480;
	d3dPP.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dPP.BackBufferFormat = d3dDM.Format;
	d3dPP.EnableAutoDepthStencil = TRUE;
	d3dPP.AutoDepthStencilFormat = D3DFMT_D16;
	d3dPP.BackBufferCount = 1;
	d3dPP.Flags |= D3DPRESENTFLAG_LOCKABLE_BACKBUFFER;

    if (hr = m_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, GetSafeHwnd(), D3DCREATE_HARDWARE_VERTEXPROCESSING, &d3dPP, &m_pDeviceD3D), FAILED(hr))
	{
		if (hr = m_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, GetSafeHwnd(), D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dPP, &m_pDeviceD3D), FAILED(hr))
		{
			if (hr = m_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_REF, GetSafeHwnd(), D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dPP, &m_pDeviceD3D), FAILED(hr))
			{
				return hr;
			}
		}
	}
	return S_OK;
}

HRESULT CSceneView::InitRenderState(void)
{
	HRESULT hr;
	D3DDISPLAYMODE d3dDM;
	D3DXVECTOR3 pvTmp[3];

	m_pDeviceD3D->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
	m_pDeviceD3D->SetRenderState(D3DRS_DITHERENABLE, FALSE);
	m_pDeviceD3D->SetRenderState(D3DRS_MULTISAMPLEANTIALIAS, FALSE);
	m_pDeviceD3D->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

	D3DVIEWPORT9 vp;
	if(hr = m_pDeviceD3D->GetViewport(&vp), FAILED(hr))
		return hr;

	if (hr = m_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3dDM), FAILED(hr))
		return hr;

	float fAspect = ((float)vp.Width / (float)vp.Height) * ((d3dDM.Height * 4.0f) / (d3dDM.Width * 3.0f));
	D3DXMatrixPerspectiveFovLH(&m_mProjection, D3DX_PI / 4.0f, fAspect, 1.0f, 1000.0f);

	for (int i = 0;i < 3;i++)
		D3DXVec3TransformCoord(&pvTmp[i], &m_pvView[i], &m_pmViewTrans[i]);
	D3DXMatrixLookAtLH(&m_mView, &pvTmp[0], &pvTmp[1], &pvTmp[2]);

	return hr;
}

HRESULT CSceneView::InitHLSL(void)
{
	HRESULT	hr;

	if (hr = D3DXCreateEffectFromFile(m_pDeviceD3D, "Viewer.fx",
					NULL, NULL, 0, NULL, &m_pEffect, NULL), FAILED(hr))
		return hr;
	
	m_hTechPhong = m_pEffect->GetTechniqueByName("PhongShader");
	if (m_hTechPhong == NULL)
		return E_FAIL;

	if (m_hWorld = m_pEffect->GetParameterByName(NULL, "g_mWorld"), m_hWorld == NULL)
		return E_FAIL;
	if (m_hWorldIT = m_pEffect->GetParameterByName(NULL, "g_mWorldIT"), m_hWorldIT == NULL)
		return E_FAIL;
	if (m_hWorldViewProj = m_pEffect->GetParameterByName(NULL, "g_mWorldViewProj"), m_hWorldViewProj == NULL)
		return E_FAIL;
	if (m_hEyePos = m_pEffect->GetParameterByName(NULL, "g_vEyePos"), m_hEyePos == NULL)
		return E_FAIL;

	D3DXVECTOR4 vTmp(m_pvView->x, m_pvView->y, m_pvView->z, 1.0f);
	m_pEffect->SetVector(m_hEyePos, &vTmp);

	return hr;
}

void CSceneView::OnLButtonDown(UINT nFlags, CPoint point)
{
	m_pntDown = point;
	CView::OnLButtonDown(nFlags, point);
}

void CSceneView::OnRButtonDown(UINT nFlags, CPoint point)
{
	m_pntDown = point;
	CView::OnRButtonDown(nFlags, point);
}

void CSceneView::OnMButtonDown(UINT nFlags, CPoint point)
{
	m_pntDown = point;
	CView::OnMButtonDown(nFlags, point);
}

void CSceneView::OnMouseMove(UINT nFlags, CPoint point)
{
	D3DXMATRIX mTrans, mTmp;
	D3DXVECTOR3 pvTmp[3];
	D3DXQUATERNION qt;

	int dx = point.x - m_pntDown.x;
	int dy = point.y - m_pntDown.y;

	switch (nFlags)
	{
	case MK_SHIFT | MK_LBUTTON:
		D3DXMatrixRotationY(&mTrans, (float)dx / 400.0f);
		mTrans *= *D3DXMatrixRotationZ(&mTmp, (float)dy / 400.0f);
		m_pmViewTrans[0] = mTrans * m_pmViewTrans[0];
		m_pmViewTrans[1] = mTrans * m_pmViewTrans[1];
		m_pmViewTrans[2] = mTrans * m_pmViewTrans[2];
		break;
	case MK_SHIFT | MK_RBUTTON:
		D3DXMatrixTranslation(&mTrans, 0.0f, dy / 40.0f, -dx / 40.0f);
		m_pmViewTrans[0] = mTrans * m_pmViewTrans[0];
		m_pmViewTrans[1] = mTrans * m_pmViewTrans[1];
		break;
	case MK_SHIFT | MK_MBUTTON:
		m_pvView[0].x += -dy / 100.0f;
		if (m_pvView[0].x < 0.1f)
			m_pvView[0].x = 0.1f;
		break;
	default:
		return;
	}

	for (int i = 0;i < 3;i++)
		D3DXVec3TransformCoord(&pvTmp[i], &m_pvView[i], &m_pmViewTrans[i]);
	D3DXMatrixLookAtLH(&m_mView, &pvTmp[0], &pvTmp[1], &pvTmp[2]);

	D3DXVECTOR4 vTmp(pvTmp->x, pvTmp->y, pvTmp->z, 1.0f);
	m_pEffect->SetVector(m_hEyePos, &vTmp);

	m_pntDown = point;

	SendMessage(WM_USER_RENDER_SCENE);

	CView::OnMouseMove(nFlags, point);
}

BOOL CSceneView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt)
{
	m_pvView[0].x += static_cast<float>(zDelta) / 60.0f;
	if (m_pvView[0].x < 0.1f)
		m_pvView[0].x = 0.1f;

	D3DXVECTOR3 pvTmp[3];
	for (int i = 0;i < 3;i++)
		D3DXVec3TransformCoord(&pvTmp[i], &m_pvView[i], &m_pmViewTrans[i]);
	D3DXMatrixLookAtLH(&m_mView, &pvTmp[0], &pvTmp[1], &pvTmp[2]);

	SendMessage(WM_USER_RENDER_SCENE);

	return CView::OnMouseWheel(nFlags, zDelta, pt);
}

void CSceneView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	switch (nChar)
	{
	case VK_RIGHT:
		m_vControlParam[0] -= D3DX_PI / 50.0;
		break;
	case VK_LEFT:
		m_vControlParam[0] += D3DX_PI / 50.0;
		break;
	case VK_DOWN:
		m_vControlParam[1] -= 2.0;
		break;
	case VK_UP:
		m_vControlParam[1] += 2.0;
		break;
	}
	CView::OnKeyDown(nChar, nRepCnt, nFlags);

	SendMessage(WM_USER_RENDER_SCENE);
}

LRESULT CSceneView::OnRenderScene(WPARAM wPara, LPARAM lPara)
{
	HRESULT hr;
	m_pDeviceD3D->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0,0,0), 1.0f, 0);

	if (hr = m_pDeviceD3D->BeginScene(), FAILED(hr))
		return -1;

	m_pEffect->SetTechnique("PhongShader");
	m_pEffect->SetVector("g_vAmbiColor", &D3DXVECTOR4(0.5f, 0.5f, 0.8f, 1.0f));
	m_pEffect->SetVector("g_vSurfColor", &D3DXVECTOR4(0.5f, 0.5f, 0.8f, 1.0f));
	for (size_t i = 0;i < m_vecSampleFigures.size();++i)
		DrawSampleMotion(i);

	m_pEffect->SetVector("g_vAmbiColor", &D3DXVECTOR4(0.5f, 0.8f, 0.5f, 1.0f));
	m_pEffect->SetVector("g_vSurfColor", &D3DXVECTOR4(0.5f, 0.8f, 0.5f, 1.0f));
	DrawSkeleton();

	DrawParameters();

	if (hr = m_pDeviceD3D->EndScene(), FAILED(hr))
		return -1;

	hr = m_pDeviceD3D->Present(NULL, NULL, NULL, NULL);

	return 0;
}

/**
 * @fn void CSceneView::DrawParameters()
 * @brief p[^̉ip`^[Qbgj
 */
void CSceneView::DrawParameters()
{
	m_pEffect->SetTechnique("PhongShader");
	D3DXMATRIX ms, mt;

	m_pEffect->SetVector("g_vAmbiColor", &D3DXVECTOR4(0.8f, 0.8f, 0.5f, 1.0f));
	m_pEffect->SetVector("g_vSurfColor", &D3DXVECTOR4(0.8f, 0.8f, 0.5f, 1.0f));
	for (std::vector<CVector*>::const_iterator cit = m_vecSampleParams.begin();cit != m_vecSampleParams.end();++cit)
	{
		D3DXMatrixScaling(&mt, 5.0f, 5.0f, 5.0f);
		D3DXMatrixTranslation(&ms, 0, 0, -80.0f);
		mt *= ms;
		D3DXMatrixRotationY(&ms, (*cit)->Data(0));
		mt *= ms;
		D3DXMatrixTranslation(&ms, 0, (*cit)->Data(1), 0);
		mt *= ms;
		D3DXMatrixScaling(&ms, 0.1f, 0.1f, 0.1f);
		mt *= ms;
		DrawMeshSub(m_pMeshSphere, mt);
	}

	if (m_vControlParam.Norm() > 0)
	{
		m_pEffect->SetVector("g_vAmbiColor", &D3DXVECTOR4(1.0f, 0.3f, 0.3f, 1.0f));
		m_pEffect->SetVector("g_vSurfColor", &D3DXVECTOR4(1.0f, 0.3f, 0.3f, 1.0f));
		D3DXMatrixScaling(&mt, 10.0f, 10.0f, 10.0f);
		D3DXMatrixTranslation(&ms, 0, 0, -80.0f);
		mt *= ms;
		D3DXMatrixRotationY(&ms, m_vControlParam[0]);
		mt *= ms;
		D3DXMatrixTranslation(&ms, 0, m_vControlParam[1], 0);
		mt *= ms;
		D3DXMatrixScaling(&ms, 0.1f, 0.1f, 0.1f);
		mt *= ms;
		DrawMeshSub(m_pMeshSphere, mt);
	}
}

HRESULT CSceneView::DrawMeshSub(const LPD3DXMESH pMesh, const D3DXMATRIX &mWorld) const
{
	HRESULT hr;
	unsigned int nNumPass;
	D3DXMATRIX mTmp;

	m_pEffect->SetMatrix(m_hWorld, &mWorld);
	D3DXMatrixInverse(&mTmp, NULL, &mWorld);
	D3DXMatrixTranspose(&mTmp, &mTmp);
	m_pEffect->SetMatrix(m_hWorldIT, &mTmp);

	mTmp = mWorld * m_mView * m_mProjection;
	m_pEffect->SetMatrix(m_hWorldViewProj, &mTmp);

	if (hr = m_pEffect->Begin(&nNumPass, 0), FAILED(hr))
		return hr;

	m_pEffect->BeginPass(0);
	pMesh->DrawSubset(0);
	m_pEffect->EndPass();

	m_pEffect->End();

	return S_OK;
}

void CSceneView::DrawSkeletonSub(const CJoint *pJoint, LPD3DXMATRIXSTACK &pMatStack)
{
	if (pJoint == NULL)
		return;
	
	pMatStack->Push();
		pMatStack->MultMatrixLocal(&pJoint->GetInitialTransform());
		DrawMeshSub(m_pMeshSphere, *pMatStack->GetTop());
	pMatStack->Pop();

	pMatStack->Push();
		pMatStack->MultMatrixLocal(&pJoint->GetTransformMatrix());
		for (size_t j = 0;j < pJoint->NumChildren();j++)
			DrawSkeletonSub(pJoint->GetChild(j), pMatStack);
	pMatStack->Pop();
}

/**
 * @fn void CSceneView::DrawSkeleton()
 * @brief p̕`
 */
void CSceneView::DrawSkeleton()
{
	if (m_pFigure == NULL)
		return;

	HRESULT hr;
	LPD3DXMATRIXSTACK pMatStack;
	const CJoint *pRoot = m_pFigure->GetRoot();;

	if (hr = D3DXCreateMatrixStack(0, &pMatStack), FAILED(hr))
		return;

	pMatStack->LoadIdentity();

	pMatStack->Scale(0.1f, 0.1f, 0.1f);
	pMatStack->MultMatrixLocal(&pRoot->GetTransformMatrix());
	for (size_t j = 0;j < pRoot->NumChildren();++j)
		DrawSkeletonSub(pRoot->GetChild(j), pMatStack);

	pMatStack->Release();
}

/**
 * @fn void CSceneView::DrawSampleMotion(size_t i)
 * @brief Tvp̕`
 */
void CSceneView::DrawSampleMotion(size_t i)
{
	if (m_vecSampleFigures[i] == NULL)
		return;

	HRESULT hr;
	LPD3DXMATRIXSTACK pMatStack;
	const CJoint *pRoot = m_vecSampleFigures[i]->GetRoot();;

	if (hr = D3DXCreateMatrixStack(0, &pMatStack), FAILED(hr))
		return;

	pMatStack->LoadIdentity();
	pMatStack->Scale(0.1f, 0.1f, 0.1f);
	pMatStack->Translate(((i + 1) % 4) * -10.0f, 0.0f, ((i + 1) / 4) * 10.0f);
	pMatStack->MultMatrixLocal(&pRoot->GetTransformMatrix());
	for (size_t j = 0;j < pRoot->NumChildren();++j)
		DrawSkeletonSub(pRoot->GetChild(j), pMatStack);

	pMatStack->Release();
}

void CSceneView::OnTimer(UINT_PTR nIDEvent)
{
	size_t cnt = 0;
	if (!m_bPlay)
		return;

	for (size_t i = 0;i < m_vecSampleFigures.size();++i)
	{
		if (m_frame >= m_vecSampleMotions[i]->NumFrames())
			m_vecSampleFigures[i]->SetPose(m_vecSampleMotions[i], m_vecSampleMotions[i]->NumFrames() - 1);
		else
		{
			m_vecSampleFigures[i]->SetPose(m_vecSampleMotions[i], m_frame);
			++cnt;
		}
	}
	if (m_pMotion != NULL)
	{
		if (m_frame >= m_pMotion->NumFrames())
			m_pFigure->SetPose(m_pMotion, m_pMotion->NumFrames() - 1);
		else
		{
			m_pFigure->SetPose(m_pMotion, m_frame);
			++cnt;
		}
	}

	m_frame = cnt == 0 ? 0 : m_frame + 2;
	SendMessage(WM_USER_RENDER_SCENE);

	CView::OnTimer(nIDEvent);
}

void CSceneView::OnFileOpen()
{
	CFileDialog dlg(TRUE, "bvh", NULL, OFN_ALLOWMULTISELECT | OFN_NOCHANGEDIR, "BVH Motion File (*.bvh)|*.bvh|||", this);
	dlg.m_ofn.lpstrFile = new char[512*1024-1];
	memset(dlg.m_ofn.lpstrFile, 0, 512*1024-1);
	dlg.m_ofn.nMaxFile = 512 * 1024 - 1;
	if (dlg.DoModal() != IDOK)
	{
		delete[] dlg.m_ofn.lpstrFile;
		return;
	}
	ReleaseMotions();

	POSITION pos = dlg.GetStartPosition();
	while (pos != NULL)
	{
		m_vecSampleFigures.push_back(new CFigure);
		m_vecSampleMotions.push_back(new CMotionData);
		NMotionFile::LoadBvhFile(dlg.GetNextPathName(pos), m_vecSampleFigures.back(), m_vecSampleMotions.back());
		m_vecSampleFigures.back()->SetPose(m_vecSampleMotions.back(), 0);
	}
	delete[] dlg.m_ofn.lpstrFile;

	SendMessage(WM_USER_RENDER_SCENE);
}

void CSceneView::OnFileSaveAs()
{
	CFileDialog dlg(TRUE, "bvh", NULL, 0, "BVH Motion File (*.bvh)|*.bvh||", this);

	if (m_pFigure == NULL || dlg.DoModal() != IDOK)
		return;

	NMotionFile::StoreBvhFile(dlg.GetPathName(), m_pFigure, m_pMotion);
}

void CSceneView::OnFilePlay()
{
	m_bPlay = !m_bPlay;
}

void CSceneView::OnUpdateFilePlay(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck(m_bPlay);
}

void transform_entire_motion(CMotionData &mot, D3DXMATRIX adjust)
{
	D3DXVECTOR3 ps, pt;
	D3DXQUATERNION qs, qt;
	D3DXMATRIX ms, mt;
	for (size_t f = 0;f < mot.NumFrames();++f)
	{
		D3DXVECTOR3 ps = mot.GetPosition(f);
		D3DXVec3TransformCoord(&pt, &ps, &adjust);
		mot.SetPosition(f, pt);

		qs = mot.GetRotation(f, 0);
		D3DXMatrixRotationQuaternion(&ms, &qs);
		mt = ms * adjust;
		mt._41 = 0; mt._42 = 0; mt._43 = 0;
		D3DXQuaternionRotationMatrix(&qt, &mt);
		mot.SetRotation(f, 0, qt);
	}
}

/**
 * @fn void CSceneView::OnMotionSpatialAlign()
 * @brief Tv[V̋ԓIK
 */
void CSceneView::OnMotionSpatialAlign()
{
	D3DXMATRIX mAdjust, ms, mt;
	D3DXVECTOR3 vAdjust, vs, vt;
	D3DXQUATERNION qs, qt;

	// ̏[gʒu_Ɉv镽sړgXtH[i͏j
	vAdjust = m_vecSampleMotions[0]->GetPosition(0);
	D3DXMatrixTranslation(&mAdjust, -vAdjust.x, 0.0f, -vAdjust.z);
	// ̃[J+X[h+XɈv]gXtH[
	vs = D3DXVECTOR3(1.0f, 0.0f, 0.0f);
	qs = m_vecSampleMotions[0]->GetRotation(0, 0);
	D3DXMatrixRotationQuaternion(&ms, &qs);
	D3DXVec3TransformCoord(&vt, &vs, &ms);
	D3DXMatrixRotationY(&mt, atan2f(vt.z, vt.x));
	mAdjust *= mt;
	// St[𓯈̃gXtH[ŕ␳
	transform_entire_motion(*m_vecSampleMotions[0], mAdjust);

	// ɍ킹đ̃TvʒuCϊ
	m_vecSampleFigures[0]->SetPose(m_vecSampleMotions[0], 0);
	std::vector<D3DXVECTOR3> pc0 = m_vecSampleFigures[0]->GetPointCloud();
	for (size_t s = 1;s < m_vecSampleFigures.size();++s)
	{
		m_vecSampleFigures[s]->SetPose(m_vecSampleMotions[s], 0);
		std::vector<D3DXVECTOR3> pc1 = m_vecSampleFigures[s]->GetPointCloud();
		mAdjust = NFigureUtil::CalcTransformBetweenFigures(pc0, pc1);
		transform_entire_motion(*m_vecSampleMotions[s], mAdjust);

		m_vecSampleFigures[s]->SetPose(m_vecSampleMotions[s], 0);
	}

	SendMessage(WM_USER_RENDER_SCENE);
}

/**
 * @fn void CSceneView::OnMotionTemporalAlign()
 * @brief Tv[V̎ԓIK
 */
void CSceneView::OnMotionTemporalAlign()
{
	CWaitCursor cur;

	while (!m_vecSampleITW.empty())
	{
		delete m_vecSampleITW.back();
		m_vecSampleITW.pop_back();
	}
	m_vecSampleITW.push_back(new CVector(m_vecSampleMotions[0]->NumFrames(), 1.0));

	CMotionData dest;
	for (size_t s = 1;s < m_vecSampleMotions.size();++s)
	{
		m_vecSampleITW.push_back(new CVector);
		NMotionSignal::Synchronize(dest, *m_vecSampleITW.back(), *m_vecSampleMotions[s], *m_vecSampleMotions[0], *m_vecSampleFigures[0]);
		*m_vecSampleMotions[s] = dest;
	}
	SendMessage(WM_USER_RENDER_SCENE);
}

CVector CSceneView::NormalizeParameter(const CVector &src)
{
	CVector dest = src;
	dest[0] = (src[0] - m_vParaRange[0][0]) / (m_vParaRange[0][1] - m_vParaRange[0][0]);
	dest[1] = (src[1] - m_vParaRange[1][0]) / (m_vParaRange[1][1] - m_vParaRange[1][0]);
	return dest;
}

/**
 * @fn void CSceneView::OnMotionCalcParameters()
 * @brief Tv[V̓p[^Zo
 */
void CSceneView::OnMotionCalcParameters()
{
	CWaitCursor cur;

	while (!m_vecSampleParams.empty())
	{
		delete m_vecSampleParams.back();
		m_vecSampleParams.pop_back();
	}

	m_vParaRange[0][0] = DBL_MAX;
	m_vParaRange[0][1] = DBL_MIN;
	m_vParaRange[1][0] = DBL_MAX;
	m_vParaRange[1][1] = DBL_MIN;

	size_t eid = m_vecSampleFigures[0]->FindNode("r_wrist_tip");
	for (size_t s = 0;s < m_vecSampleFigures.size();++s)
	{
		m_vecSampleParams.push_back(new CVector(2));

		m_vecSampleFigures[s]->SetPose(m_vecSampleMotions[s], m_vecSampleMotions[s]->NumFrames() / 2);
		D3DXVECTOR3 pos = m_vecSampleFigures[s]->GetWorldPosition(eid);
		m_vecSampleParams.back()->Data(1) = static_cast<double>(pos.y);

		m_vecSampleFigures[s]->SetPose(m_vecSampleMotions[s], 0);
		pos -= m_vecSampleFigures[s]->GetRootPosition();
		m_vecSampleParams.back()->Data(0) = static_cast<double>(atan2f(-pos.x, -pos.z));

		m_vParaRange[0][0] = GSL_MIN(m_vParaRange[0][0], m_vecSampleParams.back()->Data(0));
		m_vParaRange[0][1] = GSL_MAX(m_vParaRange[0][1], m_vecSampleParams.back()->Data(0));
		m_vParaRange[1][0] = GSL_MIN(m_vParaRange[1][0], m_vecSampleParams.back()->Data(1));
		m_vParaRange[1][1] = GSL_MAX(m_vParaRange[1][1], m_vecSampleParams.back()->Data(1));
	}

	if (m_pFigure != NULL)
		delete m_pFigure;
	m_pFigure = new CFigure(*m_vecSampleFigures.front());

	if (m_pMotion != NULL)
		delete m_pMotion;
	m_pMotion = new CMotionData(*m_vecSampleMotions.front());
	m_pFigure->SetPose(m_pMotion, 0);

	m_vControlParam[0] = (m_vParaRange[0][0] + m_vParaRange[0][1]) * 0.5;
	m_vControlParam[1] = (m_vParaRange[1][0] + m_vParaRange[1][1]) * 0.5;
	
	SendMessage(WM_USER_RENDER_SCENE);
}

/**
 * @fn void CSceneView::OnMotionAnalyze()
 * @brief Tv[V̑֍s̎Zo
 */
void CSceneView::OnMotionAnalyze()
{
	if (m_pKrigingSystem != NULL)
		delete m_pKrigingSystem;
	m_pKrigingSystem = new CMatrix(m_vecSampleParams.size() + 1, m_vecSampleParams.size() + 1, 1.0);

	for (size_t r = 0;r < m_pKrigingSystem->Row() - 1;++r)
	{
		CVector vr = NormalizeParameter(*m_vecSampleParams[r]);
		for (size_t c = r;c < m_pKrigingSystem->Col() - 1;++c)
		{
			CVector vc = NormalizeParameter(*m_vecSampleParams[c]);
			m_pKrigingSystem->Data(r, c) = (vr - vc).Norm();
			m_pKrigingSystem->Data(c, r) = m_pKrigingSystem->Data(r, c);
		}
	}
	m_pKrigingSystem->Data(m_pKrigingSystem->Row() - 1, m_pKrigingSystem->Col() - 1) = 0.0;
	*m_pKrigingSystem = m_pKrigingSystem->InverseLUD();

	SendMessage(WM_USER_RENDER_SCENE);
}

/**
 * @fn void CSceneView::OnMotionSynthesis()
 * @brief ̍
 */
void CSceneView::OnMotionSynthesis()
{
	CVector cv(m_vecSampleParams.size() + 1, 1.0);
	CVector npara = NormalizeParameter(m_vControlParam);
	for (size_t s = 0;s < m_vecSampleParams.size();++s)
	{
		CVector vp = NormalizeParameter(*m_vecSampleParams[s]);
		cv[s] = (vp - npara).Norm();
	}

	CVector weight = *m_pKrigingSystem * cv;
	CVector itw(m_vecSampleITW.front()->Size(), 0);
	std::vector<float> kernel(weight.Size() - 1);
	for (size_t s = 0;s < kernel.size();++s)
	{
		kernel[s] = weight[s];
		itw += *m_vecSampleITW[s] * kernel[s];
	}

	CMotionData dest;
	NMotionSignal::Interpolate(dest, m_vecSampleMotions, kernel);
	NMotionSignal::IncrementalTimewarp(*m_pMotion, dest, itw);
	m_pFigure->SetPose(m_pMotion, m_frame);

	SendMessage(WM_USER_RENDER_SCENE);
}
